@@ -37,10 +37,6 @@ public:
     }
 
     Rating ratingForApplication(AbstractResource *resource) const override;
-    bool hasCredentials() const override
-    {
-        return false;
-    }
     bool supportsNameChange() const override
     {
         return true;
