@@ -178,6 +178,7 @@ private:
     AppPackageKitResource *addComponent(const AppStream::Component &component) const;
     void updateProxy();
     void foundNewMajorVersion(const AppStream::Release &release);
+    void setRefresher(PackageKit::Transaction *refresh);
 
     QScopedPointer<AppStream::ConcurrentPool> m_appdata;
     bool m_appdataLoaded = false;
@@ -202,5 +203,4 @@ private:
     QPointer<PKResolveTransaction> m_resolveTransaction;
     QStringList m_globalHints;
     bool m_allPackagesLoaded = false;
-    DiscoverAction *m_reportToDistroAction;
 };
